<?php
/**
 * @author Mat <mat_sf@users.sourceforge.net>
 */
class DBIterator implements Iterator {
    private $csvFileIterator;
    private $key;
    private $begin;
    private $end;

    function __construct(CSVFileIterator $csvFileIterator, $beginTimestamp, $endTimestamp) {
        $this->csvFileIterator = $csvFileIterator;
        $this->begin = $beginTimestamp;
        $this->end = $endTimestamp;
    }

    /**
     * This methods return the current CSV row data.
     *
     * @access public
     * @return DBRecord
     */
    public function current() {
        return $this->current;
    }

    public function key() {
        return $this->key;
    }
    
    /**
     * This method move the file pointer to the next row.
     *
     * @access public
     */
    public function next() {
        $this->read();
        $this->key++;
    }

    /**
     * This method returns to first record.
     *
     * @access public
     */
    public function rewind() {
        $this->csvFileIterator->rewind();
        $this->read();
        $this->key = 0;
    }

    /**
     * This method checks if the current row is readable.
     *
     * @access public
     * @return boolean If the current row is readable.
     */
    public function valid() {
        if ($this->current === false) {
            return false;
        }
        return true;
    }

    /**
     * This method read the next row of the CSV db.
     *
     * @access private
     */
    private function read() {
        $this->get();
        //if nothing read, move to next file, if there is no other file, return
        if ($this->current===false) {
            $this->csvFileIterator->next();
            if (!$this->csvFileIterator->valid()) {
                $this->current=false;
                return;
            }
            $this->get();
        }

        if ($this->current->getTimestamp() >= $this->end) {
            $this->current=false;
            return;
        }

        return;
    }

    private function get() {
        if (!$this->csvFileIterator->valid()) {
            $this->current = false;
            return;
        }

        do {
            $record = fgetcsv($this->csvFileIterator->current());
            if ($record === false) {
                $this->current = false;
                return;
            }

            $dbRecord = DBRecordArrayMapper::mapToDBRecord($record);
        } while($dbRecord->getTimestamp() < $this->begin);

         $this->current = $dbRecord;
    }
}
?>
